
# q-expansion of newform 1063.2.a.b, downloaded from the LMFDB on 23 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1063
weight = 2
poly_data = [-1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0, 1], [1, -2], [2, -2], [-3, 0], [5, 0], [3, 1], [-1, 2], [-2, -2], [0, -1], [1, 4], [5, 2], [8, 2], [7, -5], [6, 3], [-2, -1], [1, 3], [0, -4], [-6, 4], [4, -2], [-7, -1], [0, 0], [2, 1], [-9, 1], [-1, 5], [12, -8], [-3, 5], [11, -7], [-1, -2], [-2, -9], [7, -1], [-4, 15], [-7, 5], [-1, 3], [-20, 5], [16, -2], [-11, 7], [-8, 18], [-3, -4], [-3, -2], [-13, 13], [-9, 12], [12, 5], [19, 0], [19, -6], [3, -2], [-5, -10], [-8, 13], [-4, 9], [-1, -9], [-25, 3], [1, -3], [3, 6], [2, -3], [-11, 24], [1, 13], [-12, 7], [10, -19], [-28, 5], [3, 5], [19, -6], [6, 15], [1, -14], [-1, -7], [-24, 8], [-20, 10], [-9, 19], [20, -23], [19, 7], [-8, 23], [-22, -4], [15, 8], [-26, 6], [-21, -4], [-21, 6], [27, -13], [-10, 3], [21, -15], [13, -12], [24, 0], [4, 4], [17, 13], [-14, 7], [13, 2], [26, -8], [-1, -9], [-25, -10], [-17, 11], [4, 11], [3, -4], [20, -23], [12, -20], [-18, -12], [21, -9], [14, -25], [12, -32], [-25, -2], [1, -12], [7, -12], [5, 20], [13, -10], [21, 2], [-24, 29], [-18, -3], [27, -14], [-16, -3], [-11, -10], [5, -10], [-12, -6], [-11, 24], [31, -26], [-10, -13], [-11, -1], [2, -3], [-6, -19], [-14, 32], [18, 17], [14, -16], [40, 1], [17, -28], [1, 26], [14, 6], [17, 16], [21, -15], [32, -18], [3, -32], [-34, 9], [19, -33], [9, -12], [11, 6], [1, -20], [16, 20], [3, 31], [-28, -8], [18, -36], [28, -5], [1, 7], [0, 32], [-6, 9], [17, 5], [-19, 30], [-17, 24], [26, -36], [-20, -22], [-46, 14], [-9, 15], [-26, 11], [17, -27], [27, -6], [-17, 19], [9, 14], [-24, 13], [12, -34], [-19, 36], [-5, -9], [-1, -16], [-5, -28], [-29, -15], [-17, 26], [26, -26], [5, 32], [51, -1], [29, -15], [-1, -28], [-9, 7], [1, -17], [31, -14], [-13, 38], [22, -21], [-5, -20], [-10, 9], [47, -15], [-34, 10], [3, -32], [38, -36], [31, -6], [-17, -6], [-10, 31], [4, -8], [1, 0], [0, -15], [32, -3], [-5, 3], [-48, 1], [13, -5], [11, -44], [-11, -9], [7, -26], [-15, 18], [28, -10], [-35, 48], [-19, -4], [-40, 24], [39, 15], [32, 22], [-54, 3], [29, 4], [-26, 6], [-12, -27], [-10, 12], [19, 5], [-50, 18], [-19, 8], [30, -16], [-11, 34], [17, 5], [-4, 24], [-30, 53], [30, -6], [55, -11], [-47, 20], [16, -33], [45, -4], [-2, 12], [53, -8], [11, 32], [26, -25], [19, -29], [-8, -4], [22, 28], [-49, 12], [-37, 34], [5, -12], [13, -2], [15, 17], [24, -5], [-39, 35], [35, -24], [-46, 37], [-23, -15], [36, -7], [-12, 12], [23, 2], [-9, -24], [15, -18], [-26, 1], [-12, 21], [-38, 16], [-8, 2], [28, -35], [14, 6], [-32, 0], [-15, 32], [-31, 59], [29, -41], [29, -6], [16, -46], [-16, 11], [-49, 3], [-42, 4], [45, 5], [-28, 10], [-12, -34], [-17, -7], [-59, 0], [-39, 47], [-9, -6], [-34, 11], [5, -12], [29, 2], [-19, 19], [16, 26], [8, -24], [-51, 6], [-3, -26], [45, -33], [45, -17], [-13, -4], [32, -53], [16, -24], [11, -34], [25, -12], [22, 17], [-39, 18], [-51, 19], [44, 5], [6, 0], [25, -61], [-55, 20], [-32, -19], [24, -26], [-57, 36], [-19, -28], [-28, -18], [41, -16], [-17, 29], [49, -21], [-24, 4], [-3, -35], [40, -33], [-36, -31], [53, -28], [45, -2], [37, -46], [-52, 22], [35, -26], [5, 31], [52, 15], [-25, -8], [-28, 32], [-16, 19], [-58, 35], [-7, -48], [31, 23], [-14, 40], [-45, 15], [-18, 8], [13, 18], [72, -6], [-67, 19], [35, 4], [57, 12], [10, 8], [-22, -27], [-39, -17], [10, 30], [-1, 51], [36, -25], [-75, 11], [-54, 39], [21, -10], [-10, -23], [-35, 51], [-5, 59], [13, -15], [2, 32], [59, -54], [-62, 22], [13, 25], [-26, 55], [59, -35], [41, 22], [-14, -14], [19, -68], [-55, -18], [70, 1], [-20, -14], [23, -16], [-15, 19], [34, 5], [48, -69], [27, -20], [-18, 45], [29, -2], [-26, -18], [-15, -42], [30, 20], [-67, -2], [55, -21], [42, -29], [-22, 50], [-8, 16], [4, -30], [-21, -22], [-59, 46], [-51, 40], [22, -32], [26, -36], [31, -16], [53, -40], [-54, 41], [62, -22], [20, -66], [-9, -13], [-4, -32], [-55, 13], [0, 24], [-21, 57], [50, 14], [13, -30], [23, 21], [41, -26], [21, 30], [-49, 11], [-37, 38], [25, -42], [-10, 27], [-61, 12], [9, 30], [77, 11], [-32, -22], [13, -34], [28, -47], [82, 10], [24, -44], [75, -22], [18, 6], [-48, 10], [20, -21], [46, 17], [-22, 64], [31, 5], [4, 10], [-34, -6], [14, -28], [8, -4], [3, 21], [-28, 1], [44, 29], [9, -14], [-88, 12], [-1, -52], [70, -18], [-8, -6], [21, -67], [43, -8], [18, -51], [-56, 1], [5, -30], [-5, 67], [-91, 15], [-77, -12], [-74, 16], [15, 28], [2, 42], [58, -18], [-28, 55], [16, -17], [-1, -27], [-90, 12], [62, -38], [-4, 23], [25, 30], [46, 7], [51, -43], [-16, 48], [-11, 66], [36, -72], [-26, 2], [28, -44], [88, -9]]
